<?php

namespace Drp\ProjectHelpers\Test;


use Drp\ProjectHelpers\Tests\Concerns\InteractsWithModels;
use Illuminate\Database\Eloquent\Model;
use PHPUnit\Framework\ExpectationFailedException;

class InteractsWithModelsTest extends TestCase
{
    /**
     * @test
     */
    public function assert_passes_when_models_and_ids_are_the_same()
    {
        $class = New Tester;
        $model = new DbModelA(['id' => 1]);
        $class->assertSameModel($model, $model);
    }

    /**
     * @test
     */
    public function assert_fails_when_models_are_the_same_but_ids_are_different()
    {
        try {
            $class = New Tester;
            $modelA = new DbModelA(['id' => 1]);
            $modelB = new DbModelA(['id' => 2]);
            $class->assertSameModel($modelA, $modelB);
        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

    /**
     * @test
     */
    public function assert_fails_when_models_are_different_but_ids_are_the_same()
    {
        try {
            $class = New Tester;
            $modelA = new DbModelA(['id' => 1]);
            $modelB = new DbModelB(['id' => 1]);
            $class->assertSameModel($modelA, $modelB);

        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

    /**
     * @test
     */
    public function assert_passes_when_models_are_not_the_same()
    {
        $class = New Tester;
        $modelA = new DbModelA(['id' => 1]);
        $modelB = new DbModelB(['id' => 1]);
        $class->assertNotSameModel($modelA, $modelB);
    }

    /**
     * @test
     */
    public function assert_passes_when_models_are_the_same_but_id_are_different()
    {
        $class = New Tester;
        $modelA = new DbModelA(['id' => 1]);
        $modelB = new DbModelA(['id' => 2]);
        $class->assertNotSameModel($modelA, $modelB);
    }

    /**
     * @test
     */
    public function assert_fails_when_models_and_ids_are_the_same()
    {
        try {
            $class = New Tester;
            $modelA = new DbModelA(['id' => 1]);
            $modelB = new DbModelA(['id' => 1]);
            $class->assertNotSameModel($modelA, $modelB);

        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

    /**
     * @test
     */
    public function assert_passes_when_collection_contains_model()
    {
        $class = New Tester;
        $model = new DbModelA(['id' => 1]);
        $class->assertContainsModel($model, collect([$model]));
    }

    /**
     * @test
     */
    public function assert_passes_when_collection_does_not_contains_model()
    {
        $class = New Tester;
        $modelA = new DbModelA(['id' => 1]);
        $modelB = new DbModelA(['id' => 2]);
        $class->assertNotContainsModel($modelA, collect([$modelB]));
    }

    /**
     * @test
     */
    public function assert_fails_when_collection_contains_model()
    {
        try {
            $class = New Tester;
            $model = new DbModelA(['id' => 1]);
            $class->assertNotContainsModel($model, collect([$model]));
        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

    /**
     * @test
     */
    public function assert_fails_when_collection_does_not_contains_model()
    {
        try {
            $class = New Tester;
            $modelA = new DbModelA(['id' => 1]);
            $modelB = new DbModelA(['id' => 2]);
            $class->assertContainsModel($modelA, collect([$modelB]));
        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }


    /**
     * @test
     */
    public function assert_passes_when_array_contains_model()
    {
        $class = New Tester;
        $model = new DbModelA(['id' => 1]);
        $class->assertContainsModel($model, [$model]);
    }

    /**
     * @test
     */
    public function assert_passes_when_array_does_not_contains_model()
    {
        $class = New Tester;
        $modelA = new DbModelA(['id' => 1]);
        $modelB = new DbModelA(['id' => 2]);
        $class->assertNotContainsModel($modelA, [$modelB]);
    }

    /**
     * @test
     */
    public function assert_fails_when_array_contains_model()
    {
        try {
            $class = New Tester;
            $model = new DbModelA(['id' => 1]);
            $class->assertNotContainsModel($model, [$model]);
        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

    /**
     * @test
     */
    public function assert_fails_when_array_does_not_contains_model()
    {
        try {
            $class = New Tester;
            $modelA = new DbModelA(['id' => 1]);
            $modelB = new DbModelA(['id' => 2]);
            $class->assertContainsModel($modelA, [$modelB]);
        } catch (ExpectationFailedException $e) {
            return;
        }

        $this->fail('Assertion did not thrown an exception');
    }

}


Class DbModelA extends Model
{
    protected $fillable = ['id'];
}


Class DbModelB extends Model
{
    protected $fillable = ['id'];
}


Class Tester
{
    Use InteractsWithModels;
}

